@@ -34,6 +34,8 @@ extern "C" {
   in the string specified by `str_ptr' and `str_length'. If the starting bytes
   are invalid as a character, grn_tokenizer_charlen() returns 0. See
   grn_encoding in "groonga.h" for more details of `encoding'
+
+  Deprecated. Use grn_plugin_charlen() instead.
  */
 int grn_tokenizer_charlen(grn_ctx *ctx, const char *str_ptr,
                           unsigned int str_length, grn_encoding encoding);
@@ -42,6 +44,8 @@ int grn_tokenizer_charlen(grn_ctx *ctx, const char *str_ptr,
   grn_tokenizer_isspace() returns the length (#bytes) of the first character
   in the string specified by `str_ptr' and `str_length' if it is a space
   character. Otherwise, grn_tokenizer_isspace() returns 0.
+
+  Deprecated. Use grn_plugin_isspace() instead.
  */
 int grn_tokenizer_isspace(grn_ctx *ctx, const char *str_ptr,
                           unsigned int str_length, grn_encoding encoding);
@@ -78,18 +82,24 @@ struct _grn_tokenizer_query {
   const char *ptr;
   unsigned int length;
   grn_encoding encoding;
+  unsigned int flags;
+  grn_bool have_tokenized_delimiter;
+  unsigned int token_mode;
 };
 
 /*
   grn_tokenizer_query_open() parses `args' and returns a new object of
   grn_tokenizer_query. The new object stores information of the query.
-  grn_tokenizer_query_create() normalizes the query if the target table
-  requires normalization. grn_tokenizer_query_create() returns NULL if
-  something goes wrong. Note that grn_tokenizer_query_create() must be called
+  grn_tokenizer_query_open() normalizes the query if the target table
+  requires normalization. grn_tokenizer_query_open() returns NULL if
+  something goes wrong. Note that grn_tokenizer_query_open() must be called
   just once in the function that initializes a tokenizer.
+
+  See `GRN_STRING_*' flags for `normalize_flags'.
  */
 GRN_PLUGIN_EXPORT grn_tokenizer_query *grn_tokenizer_query_open(grn_ctx *ctx,
-                                                                int num_args, grn_obj **args);
+                                                                int num_args, grn_obj **args,
+                                                                unsigned int normalize_flags);
 
 /*
   grn_tokenizer_query_create() is deprecated. Use grn_tokenizer_query_open()
@@ -136,16 +146,35 @@ GRN_PLUGIN_EXPORT void grn_tokenizer_token_init(grn_ctx *ctx, grn_tokenizer_toke
 GRN_PLUGIN_EXPORT void grn_tokenizer_token_fin(grn_ctx *ctx, grn_tokenizer_token *token);
 
 /*
-  grn_tokenizer_status provides a list of tokenizer status codes.
-  GRN_TOKENIZER_CONTINUE means that the next token is not the last one and
-  GRN_TOKENIZER_LAST means that the next token is the last one. If a document
-  or query contains no tokens, please push an empty string with
-  GRN_TOKENIZER_LAST as a token.
+ * grn_tokenizer_status is a flag set for tokenizer status codes.
+ * If a document or query contains no tokens, push an empty string with
+ * GRN_TOKENIZER_TOKEN_LAST as a token.
  */
-typedef enum {
-  GRN_TOKENIZER_CONTINUE = 0,
-  GRN_TOKENIZER_LAST     = 1
-} grn_tokenizer_status;
+typedef unsigned int grn_tokenizer_status;
+
+/* GRN_TOKENIZER_TOKEN_CONTINUE means that the next token is not the last one. */
+#define GRN_TOKENIZER_TOKEN_CONTINUE           (0)
+/* GRN_TOKENIZER_TOKEN_LAST means that the next token is the last one. */
+#define GRN_TOKENIZER_TOKEN_LAST               (0x01L<<0)
+/* GRN_TOKENIZER_TOKEN_OVERLAP means that ... */
+#define GRN_TOKENIZER_TOKEN_OVERLAP            (0x01L<<1)
+/* GRN_TOKENIZER_TOKEN_UNMATURED means that ... */
+#define GRN_TOKENIZER_TOKEN_UNMATURED          (0x01L<<2)
+/* GRN_TOKENIZER_TOKEN_REACH_END means that ... */
+#define GRN_TOKENIZER_TOKEN_REACH_END          (0x01L<<3)
+/* GRN_TOKENIZER_TOKEN_SKIP means that the token is skipped */
+#define GRN_TOKENIZER_TOKEN_SKIP               (0x01L<<4)
+/* GRN_TOKENIZER_TOKEN_SKIP_WITH_POSITION means that the token and postion is skipped */
+#define GRN_TOKENIZER_TOKEN_SKIP_WITH_POSITION (0x01L<<5)
+
+/*
+ * GRN_TOKENIZER_CONTINUE and GRN_TOKENIZER_LAST are deprecated. They
+ * are just for backward compatibility. Use
+ * GRN_TOKENIZER_TOKEN_CONTINUE and GRN_TOKENIZER_TOKEN_LAST
+ * instead.
+ */
+#define GRN_TOKENIZER_CONTINUE GRN_TOKENIZER_TOKEN_CONTINUE
+#define GRN_TOKENIZER_LAST     GRN_TOKENIZER_TOKEN_LAST
 
 /*
   grn_tokenizer_token_push() pushes the next token into `token'. Note that
