@@ -23,6 +23,7 @@
 #include "io.h"
 #include "dat.h"
 #include "util.h"
+#include "normalizer_in.h"
 
 #include "dat/trie.hpp"
 #include "dat/cursor-factory.hpp"
@@ -303,11 +304,11 @@ grn_dat_create(grn_ctx *ctx, const char *path, uint32_t,
   dat->header->file_id = 0;
   if (dat->header->flags & GRN_OBJ_KEY_NORMALIZE) {
     dat->header->flags &= ~GRN_OBJ_KEY_NORMALIZE;
-    dat->header->normalizer = GRN_DB_NORMALIZER_AUTO;
-    dat->normalizer = grn_ctx_at(ctx, dat->header->normalizer);
+    dat->normalizer = grn_ctx_get(ctx, GRN_NORMALIZER_AUTO_NAME, -1);
+    dat->header->normalizer = grn_obj_id(ctx, dat->normalizer);
   } else {
-    dat->header->normalizer = GRN_ID_NIL;
     dat->normalizer = NULL;
+    dat->header->normalizer = GRN_ID_NIL;
   }
   dat->encoding = encoding;
   dat->tokenizer = NULL;
@@ -345,13 +346,15 @@ grn_dat_open(grn_ctx *ctx, const char *path)
   }
   dat->file_id = dat->header->file_id;
   dat->encoding = dat->header->encoding;
-  dat->obj.header.flags = dat->header->flags;
   dat->tokenizer = grn_ctx_at(ctx, dat->header->tokenizer);
   if (dat->header->flags & GRN_OBJ_KEY_NORMALIZE) {
     dat->header->flags &= ~GRN_OBJ_KEY_NORMALIZE;
-    dat->header->normalizer = GRN_DB_NORMALIZER_AUTO;
+    dat->normalizer = grn_ctx_get(ctx, GRN_NORMALIZER_AUTO_NAME, -1);
+    dat->header->normalizer = grn_obj_id(ctx, dat->normalizer);
+  } else {
+    dat->normalizer = grn_ctx_at(ctx, dat->header->normalizer);
   }
-  dat->normalizer = grn_ctx_at(ctx, dat->header->normalizer);
+  dat->obj.header.flags = dat->header->flags;
   return dat;
 }
 
@@ -461,7 +464,7 @@ grn_dat_add(grn_ctx *ctx, grn_dat *dat, const void *key,
       *added = res ? 1 : 0;
     }
     return trie->get_key(key_pos).id();
-  } catch (const grn::dat::SizeError &ex) {
+  } catch (const grn::dat::SizeError &) {
     if (!grn_dat_rebuild_trie(ctx, dat)) {
       return GRN_ID_NIL;
     }
@@ -611,7 +614,7 @@ grn_dat_update_by_id(grn_ctx *ctx, grn_dat *dat, grn_id src_key_id,
       if (!trie->update(src_key_id, dest_key, dest_key_size)) {
         return GRN_INVALID_ARGUMENT;
       }
-    } catch (const grn::dat::SizeError &ex) {
+    } catch (const grn::dat::SizeError &) {
       if (!grn_dat_rebuild_trie(ctx, dat)) {
         return ctx->rc;
       }
@@ -646,7 +649,7 @@ grn_dat_update(grn_ctx *ctx, grn_dat *dat,
       if (!trie->update(src_key, src_key_size, dest_key, dest_key_size)) {
         return GRN_INVALID_ARGUMENT;
       }
-    } catch (const grn::dat::SizeError &ex) {
+    } catch (const grn::dat::SizeError &) {
       if (!grn_dat_rebuild_trie(ctx, dat)) {
         return ctx->rc;
       }
@@ -685,7 +688,7 @@ grn_dat_scan(grn_ctx *ctx, grn_dat *dat, const char *str,
     return 0;
   }
 
-  int num_scan_hits = 0;
+  unsigned int num_scan_hits = 0;
   try {
     if (dat->normalizer) {
       int flags = GRN_STRING_WITH_CHECKS;
@@ -772,7 +775,7 @@ grn_dat_scan(grn_ctx *ctx, grn_dat *dat, const char *str,
         "grn::dat::lcp_search failed");
     return -1;
   }
-  return num_scan_hits;
+  return static_cast<int>(num_scan_hits);
 }
 
 grn_id
