@@ -50,9 +50,6 @@
 #ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
 #endif /* HAVE_SYS_TIME_H */
-#ifdef HAVE_SYS_TIMEB_H
-#include <sys/timeb.h>
-#endif /* HAVE_SYS_TIMEB_H */
 
 #ifdef HAVE_SYS_RESOURCE_H
 #include <sys/resource.h>
