@@ -17,6 +17,7 @@
 #include "groonga_in.h"
 #include <limits.h>
 #include <stdio.h>
+#include <stdarg.h>
 #include <string.h>
 #include "db.h"
 #include "str.h"
@@ -26,6 +27,11 @@
 #endif /* _ISOC99_SOURCE */
 #include <math.h>
 
+/* For Visual C++ 2010. Drop the code when we drop Visual C++ 2010 support. */
+#if defined(_MSC_VER) && _MSC_VER < 1800
+# define va_copy(destination, source) destination = source
+#endif
+
 inline static int
 grn_str_charlen_utf8(grn_ctx *ctx, const unsigned char *str, const unsigned char *end)
 {
@@ -1994,7 +2000,8 @@ grn_bulk_truncate(grn_ctx *ctx, grn_obj *bulk, unsigned int len)
     if (GRN_BULK_BUFSIZE < len) {
       return grn_bulk_space_clear(ctx, bulk, len);
     } else {
-      bulk->header.flags = len;
+      bulk->header.flags &= ~GRN_BULK_BUFSIZE_MAX;
+      bulk->header.flags += len;
     }
   }
   return GRN_SUCCESS;
@@ -2463,6 +2470,52 @@ grn_text_time2rfc1123(grn_ctx *ctx, grn_obj *bulk, int sec)
   return GRN_SUCCESS;
 }
 
+grn_rc
+grn_text_printf(grn_ctx *ctx, grn_obj *bulk, const char *format, ...)
+{
+  va_list args;
+
+  va_start(args, format);
+  grn_text_vprintf(ctx, bulk, format, args);
+  va_end(args);
+
+  return GRN_SUCCESS;
+}
+
+grn_rc
+grn_text_vprintf(grn_ctx *ctx, grn_obj *bulk, const char *format, va_list args)
+{
+  int rest_size, written_size;
+
+  {
+    va_list copied_args;
+
+    rest_size = GRN_BULK_REST(bulk);
+    va_copy(copied_args, args);
+    written_size = vsnprintf(GRN_BULK_CURR(bulk), rest_size,
+                             format, copied_args);
+    va_end(copied_args);
+  }
+
+  if (written_size >= rest_size) {
+    grn_rc rc;
+    int required_size = written_size + 1; /* "+ 1" for terminate '\0'. */
+
+    rc = grn_bulk_reserve(ctx, bulk, GRN_BULK_VSIZE(bulk) + required_size);
+    if (rc) {
+      return rc;
+    }
+    written_size = vsnprintf(GRN_BULK_CURR(bulk), required_size,
+                             format, args);
+  }
+
+  if (written_size < 0) {
+    return GRN_INVALID_ARGUMENT;
+  }
+
+  GRN_BULK_INCR_LEN(bulk, written_size);
+  return GRN_SUCCESS;
+}
 
 grn_rc
 grn_bulk_fin(grn_ctx *ctx, grn_obj *buf)
@@ -2716,22 +2769,26 @@ grn_text_otoj(grn_ctx *ctx, grn_obj *bulk, grn_obj *obj, grn_obj_format *format)
         }
         GRN_TEXT_PUTC(ctx, bulk, ']');
       } else {
-        grn_obj *table = grn_ctx_at(ctx, obj->header.domain);
-        grn_id id = GRN_RECORD_VALUE(obj);
-        if (table && table->header.type != GRN_TABLE_NO_KEY) {
-          /* todo : temporal patch. grn_table_at() is kinda costful... */
-          if (grn_table_at(ctx, table, id)) {
-            grn_obj *accessor = grn_obj_column(ctx, table,
-                                               GRN_COLUMN_NAME_KEY,
-                                               GRN_COLUMN_NAME_KEY_LEN);
-            if (accessor) {
-              grn_obj_get_value(ctx, accessor, id, &buf);
-              grn_obj_unlink(ctx, accessor);
-            }
-          }
-          grn_text_otoj(ctx, bulk, &buf, format);
+        if (GRN_BULK_VSIZE(obj) == 0) {
+          GRN_TEXT_PUTS(ctx, bulk, "null");
         } else {
-          grn_text_lltoa(ctx, bulk, id);
+          grn_obj *table = grn_ctx_at(ctx, obj->header.domain);
+          grn_id id = GRN_RECORD_VALUE(obj);
+          if (table && table->header.type != GRN_TABLE_NO_KEY) {
+            /* todo : temporal patch. grn_table_at() is kinda costful... */
+            if (grn_table_at(ctx, table, id)) {
+              grn_obj *accessor = grn_obj_column(ctx, table,
+                                                 GRN_COLUMN_NAME_KEY,
+                                                 GRN_COLUMN_NAME_KEY_LEN);
+              if (accessor) {
+                grn_obj_get_value(ctx, accessor, id, &buf);
+                grn_obj_unlink(ctx, accessor);
+              }
+            }
+            grn_text_otoj(ctx, bulk, &buf, format);
+          } else {
+            grn_text_lltoa(ctx, bulk, id);
+          }
         }
       }
     }
@@ -2847,31 +2904,41 @@ grn_text_otoj(grn_ctx *ctx, grn_obj *bulk, grn_obj *obj, grn_obj_format *format)
   case GRN_VECTOR :
     if (obj->header.domain == GRN_DB_VOID) {
       ERR(GRN_INVALID_ARGUMENT, "invalid obj->header.domain");
-    }
-    if (format) {
-      ERR(GRN_FUNCTION_NOT_IMPLEMENTED,
-          "cannot print GRN_VECTOR using grn_obj_format");
     } else {
       unsigned int i, n;
       grn_obj value;
+      grn_obj weight;
+      grn_bool with_weight;
+
       GRN_VOID_INIT(&value);
+      GRN_UINT32_INIT(&weight, 0);
+      with_weight = (format && format->flags & GRN_OBJ_FORMAT_WITH_WEIGHT);
       n = grn_vector_size(ctx, obj);
       GRN_TEXT_PUTC(ctx, bulk, '[');
       for (i = 0; i < n; i++) {
         const char *_value;
-        unsigned int weight, length;
+        unsigned int _weight, length;
         grn_id domain;
         if (i) { GRN_TEXT_PUTC(ctx, bulk, ','); }
 
         length = grn_vector_get_element(ctx, obj, i,
-                                        &_value, &weight, &domain);
+                                        &_value, &_weight, &domain);
         if (domain != GRN_DB_VOID) {
           grn_obj_reinit(ctx, &value, domain, 0);
         } else {
           grn_obj_reinit(ctx, &value, obj->header.domain, 0);
         }
+        if (with_weight) {
+          GRN_TEXT_PUTC(ctx, bulk, '{');
+        }
         grn_bulk_write(ctx, &value, _value, length);
         grn_text_otoj(ctx, bulk, &value, NULL);
+        if (with_weight) {
+          GRN_TEXT_PUTC(ctx, bulk, ':');
+          GRN_UINT32_SET(ctx, &weight, _weight);
+          grn_text_otoj(ctx, bulk, &weight, NULL);
+          GRN_TEXT_PUTC(ctx, bulk, '}');
+        }
       }
       GRN_TEXT_PUTC(ctx, bulk, ']');
     }
